@@ -45,10 +45,10 @@ static char id_str[MAX_INFO_STRLEN];
  *                                                                 *
  *******************************************************************/
 
-static size_t avi_read(int fd, char *buf, size_t len)
+static ssize_t avi_read(int fd, char *buf, size_t len)
 {
-   size_t n = 0;
-   size_t r = 0;
+   ssize_t n = 0;
+   ssize_t r = 0;
 
    while (r < len) {
       n = read (fd, buf + r, len - r);
@@ -61,10 +61,10 @@ static size_t avi_read(int fd, char *buf, size_t len)
    return r;
 }
 
-static size_t avi_write (int fd, char *buf, size_t len)
+static ssize_t avi_write (int fd, char *buf, size_t len)
 {
-   size_t n = 0;
-   size_t r = 0;
+   ssize_t n = 0;
+   ssize_t r = 0;
 
    while (r < len) {
       n = write (fd, buf + r, len - r);
@@ -191,6 +191,24 @@ static int avi_add_index_entry(avi_t *AVI, unsigned char *tag, long flags, unsig
    return 0;
 }
 
+/* Returns 1 if more audio is in that video junk */
+int AVI_can_read_audio(avi_t *AVI)
+{
+   if(AVI->mode==AVI_MODE_WRITE) { return -1; }
+   if(!AVI->video_index)         { return -1; }
+   if(!AVI->track[AVI->aptr].audio_index)         { return -1; }      
+
+   // is it -1? the last ones got left out --tibit 
+   //if (AVI->track[AVI->aptr].audio_posc>=AVI->track[AVI->aptr].audio_chunks-1) {
+   if (AVI->track[AVI->aptr].audio_posc>=AVI->track[AVI->aptr].audio_chunks) {
+       return 0;
+   }
+
+   if (AVI->video_pos >= AVI->video_frames) return 1;
+   
+   if (AVI->track[AVI->aptr].audio_index[AVI->track[AVI->aptr].audio_posc].pos < AVI->video_index[AVI->video_pos].pos) return 1;
+   else return 0;
+}
 /*
    AVI_open_output_file: Open an AVI File and write a bunch
                          of zero bytes as space for the header.
@@ -544,6 +562,143 @@ int avi_update_header(avi_t *AVI)
    return 0;
 }
 
+static int valid_info_tag(char *c)
+{
+    if      (!strncmp(c, "IARL", 4)) return 1; 
+    else if (!strncmp(c, "IART", 4)) return 1; 
+    else if (!strncmp(c, "ICMS", 4)) return 1; 
+    else if (!strncmp(c, "ICMT", 4)) return 1; 
+    else if (!strncmp(c, "ICOP", 4)) return 1; 
+    else if (!strncmp(c, "ICRD", 4)) return 1; 
+    else if (!strncmp(c, "ICRP", 4)) return 1; 
+    else if (!strncmp(c, "IDIM", 4)) return 1; 
+    else if (!strncmp(c, "IDPI", 4)) return 1; 
+    else if (!strncmp(c, "IENG", 4)) return 1; 
+    else if (!strncmp(c, "IGNR", 4)) return 1; 
+    else if (!strncmp(c, "IKEY", 4)) return 1; 
+    else if (!strncmp(c, "ILGT", 4)) return 1; 
+    else if (!strncmp(c, "IMED", 4)) return 1; 
+    else if (!strncmp(c, "INAM", 4)) return 1; 
+    else if (!strncmp(c, "IPLT", 4)) return 1; 
+    else if (!strncmp(c, "IPRD", 4)) return 1; 
+    else if (!strncmp(c, "ISBJ", 4)) return 1; 
+    else if (!strncmp(c, "ISHP", 4)) return 1; 
+    else if (!strncmp(c, "ISRC", 4)) return 1; 
+    else if (!strncmp(c, "ISRF", 4)) return 1; 
+    else if (!strncmp(c, "ITCH", 4)) return 1;
+    else return 0;
+
+    return 0;
+}
+// see ../docs/avi_comments.txt
+// returns the length of written stream (-1 on error)
+static int avi_parse_comments (int fd, char *buf, int space_left) 
+{
+    int len=0, readlen=0, k;
+    char *data, *c, *d;
+    struct stat st;
+    
+    // safety checks
+    if (fd<=0 || !buf || space_left<=0)
+	return -1;
+
+    memset (buf, 0, space_left);
+    if (fstat (fd, &st) == -1) {
+	perror ("stat");
+	return -1;
+    }
+
+    if ( !(data = malloc(st.st_size*sizeof(char)+1)) ) {
+	printf("malloc failed\n"); 
+	return -1;
+    }
+
+    readlen = avi_read ( fd, data, st.st_size);
+
+    //printf("Read %d bytes from %d\n", readlen, fd);
+
+    c = data;
+    space_left--;
+    
+    while (len < space_left) {
+	if ( !c || *c == '\0')
+	    break; // eof;
+	else if (*c == '#') { // comment, ignore
+	    c = strchr(c, '\n')+1;
+	}
+	else if (*c == '\n') { // empty, ignore
+	    c++;
+	}
+	else if (*c == 'I') {
+
+	    // do not write ISFT -- this is written by transcode
+	    // and important for debugging.
+	    if (!valid_info_tag(c)) {
+		// skip this line
+		while (c && *c && *c != '\n' ) {
+		    c++;
+		}
+		continue;
+	    }
+
+	    // set d after TAG
+	    d = c+4;
+
+	    // find first non-blank (!space or !TAB)
+	    while ( d && *d && (*d == ' ' || *d == '	')) ++d;
+	    if (!d) break;
+
+	    // TAG without argument is fine but ignored
+	    if (*d == '\n' || *d == '\r') { 
+		c = d+1; 
+		if (*c == '\n') ++c; 
+		continue; 
+	    }
+ 
+	    k = 0;
+	    while (d[k] != '\r' && d[k] != '\n' && d[k] != '\0') ++k;
+	    if (k>=space_left) return len;
+
+	    // write TAG
+	    memcpy(buf+len,c,4); 
+	    len += 4;
+
+	    // write length + '\0'
+	    long2str(buf+len, k+1); len += 4;
+
+	    // write comment string
+	    memcpy (buf+len, d, k);
+	    // must be null terminated
+	    *(buf+len+k+1) = '\0';
+
+	    // PAD
+	    if ((k+1)&1) {
+		k++;
+		*(buf+len+k+1) = '\0';
+	    }
+	    len += k+1;
+  
+	    // advance c
+	    while (*c != '\n' && *c != '\0') ++c;
+	    if (*c != '\0') ++c;
+	    else break;
+
+	} else {
+
+	    // ignore junk lines
+	    while (c && *c && *c != '\n' ) {
+		if (*c == ' ' || *c == '	') { c++; break; }
+		c++;
+	    }
+	    if (!c) break;
+	}
+
+    }
+    free(data);
+
+    return len;
+}
+
 /*
   Write the header of an AVI file and close it.
   returns 0 on success, -1 on write error.
@@ -560,6 +715,8 @@ static int avi_close_output_file(avi_t *AVI)
 
 #ifdef INFO_LIST
    long info_len;
+   long id_len, real_id_len;
+   long info_start_pos;
 //   time_t calptr;
 #endif
 
@@ -673,7 +830,7 @@ static int avi_close_output_file(avi_t *AVI)
    OUTLONG(0);                  /* Start */
    OUTLONG(AVI->video_frames);  /* Length */
    OUTLONG(0);                  /* SuggestedBufferSize */
-   OUTLONG(-1);                 /* Quality */
+   OUTLONG(0);                 /* Quality */
    OUTLONG(0);                  /* SampleSize */
    OUTLONG(0);                  /* Frame */
    OUTLONG(0);                  /* Frame */
@@ -706,8 +863,15 @@ static int avi_close_output_file(avi_t *AVI)
      
      //if (AVI->track[j].a_chans && AVI->track[j].audio_bytes)
        {
+	 unsigned long nBlockAlign;
 	   
 	 sampsize = avi_sampsize(AVI, j);
+
+	 nBlockAlign = (AVI->track[j].a_rate<32000)?576:1152;
+	 /*
+	 printf("XXX sampsize (%d) block (%ld) rate (%ld) audio_bytes (%ld) chunks(%ld)\n", 
+		 sampsize, nBlockAlign, AVI->track[j].a_rate, AVI->track[j].audio_bytes, AVI->track[j].audio_chunks );
+		 */
 	   
 	 OUT4CC ("LIST");
 	 OUTLONG(0);        /* Length of list in bytes, don't know yet */
@@ -728,39 +892,82 @@ static int avi_close_output_file(avi_t *AVI)
 	 OUTLONG(0);             /* Flags */
 	 OUTLONG(0);             /* Reserved, MS says: wPriority, wLanguage */
 	 OUTLONG(0);             /* InitialFrames */
-	   
-	 // ThOe /4
-	 OUTLONG(sampsize/4);      /* Scale */
-	 OUTLONG(1000*AVI->track[j].mp3rate/8);
-	 OUTLONG(0);             /* Start */
-	 OUTLONG(4*AVI->track[j].audio_bytes/sampsize);   /* Length */
-	 OUTLONG(0);             /* SuggestedBufferSize */
-	 OUTLONG(-1);            /* Quality */
-	   
-	 // ThOe /4
-	 OUTLONG(sampsize/4);    /* SampleSize */
-	   
-	 OUTLONG(0);             /* Frame */
-	 OUTLONG(0);             /* Frame */
-	 //	 OUTLONG(0);             /* Frame */
-	 //OUTLONG(0);             /* Frame */
+	 
+	 // VBR 
+	 if (AVI->track[j].a_fmt == 0x55 && AVI->track[j].a_vbr) {
+	     OUTLONG(nBlockAlign);                   /* Scale */
+	     OUTLONG(AVI->track[j].a_rate);          /* Rate */
+	     OUTLONG(0);                             /* Start */
+	     OUTLONG(AVI->track[j].audio_chunks);    /* Length */
+	     OUTLONG(0);                             /* SuggestedBufferSize */
+	     OUTLONG(0);                             /* Quality */
+	     OUTLONG(0);                             /* SampleSize */
+	     OUTLONG(0);                             /* Frame */
+	     OUTLONG(0);                             /* Frame */
+	 } else {
+	     OUTLONG(sampsize/4);                    /* Scale */
+	     OUTLONG(1000*AVI->track[j].mp3rate/8);  /* Rate */
+	     OUTLONG(0);                             /* Start */
+	     OUTLONG(4*AVI->track[j].audio_bytes/sampsize);   /* Length */
+	     OUTLONG(0);                             /* SuggestedBufferSize */
+	     OUTLONG(0);                             /* Quality */
+	     OUTLONG(sampsize/4);                    /* SampleSize */
+	     OUTLONG(0);                             /* Frame */
+	     OUTLONG(0);                             /* Frame */
+	 }
 	   
 	 /* The audio stream format */
-	 
+
 	 OUT4CC ("strf");
-	 OUTLONG(16);                   /* # of bytes to follow */
-	 OUTSHRT(AVI->track[j].a_fmt);           /* Format */
-	 OUTSHRT(AVI->track[j].a_chans);         /* Number of channels */
-	 OUTLONG(AVI->track[j].a_rate);          /* SamplesPerSec */
-	 //ThOe
-	 OUTLONG(1000*AVI->track[j].mp3rate/8);
-	 //ThOe (/4)
 	 
-	 OUTSHRT(sampsize/4);           /* BlockAlign */
+	 if (AVI->track[j].a_fmt == 0x55 && AVI->track[j].a_vbr) {
+
+	     OUTLONG(30);                            /* # of bytes to follow */ // mplayer writes 28
+	     OUTSHRT(AVI->track[j].a_fmt);           /* Format */                  // 2
+	     OUTSHRT(AVI->track[j].a_chans);         /* Number of channels */      // 2
+	     OUTLONG(AVI->track[j].a_rate);          /* SamplesPerSec */           // 4
+	     //ThOe/tibit
+	     OUTLONG(1000*AVI->track[j].mp3rate/8);  /* maybe we should write an avg. */ // 4
+	     OUTSHRT(nBlockAlign);                   /* BlockAlign */              // 2
+	     OUTSHRT(AVI->track[j].a_bits);          /* BitsPerSample */           // 2
+
+	     OUTSHRT(12);                           /* cbSize */                   // 2
+	     OUTSHRT(1);                            /* wID */                      // 2
+	     OUTLONG(2);                            /* fdwFlags */                 // 4
+	     OUTSHRT(nBlockAlign);                  /* nBlockSize */               // 2
+	     OUTSHRT(1);                            /* nFramesPerBlock */          // 2
+	     OUTSHRT(0);                            /* nCodecDelay */              // 2
 	 
+	 } else if (AVI->track[j].a_fmt == 0x55 && !AVI->track[j].a_vbr) {
+
+	     OUTLONG(30);                            /* # of bytes to follow */
+	     OUTSHRT(AVI->track[j].a_fmt);           /* Format */
+	     OUTSHRT(AVI->track[j].a_chans);         /* Number of channels */
+	     OUTLONG(AVI->track[j].a_rate);          /* SamplesPerSec */
+	     //ThOe/tibit
+	     OUTLONG(1000*AVI->track[j].mp3rate/8);
+	     OUTSHRT(sampsize/4);                    /* BlockAlign */
+	     OUTSHRT(AVI->track[j].a_bits);          /* BitsPerSample */
+
+	     OUTSHRT(12);                           /* cbSize */
+	     OUTSHRT(1);                            /* wID */
+	     OUTLONG(2);                            /* fdwFlags */
+	     OUTSHRT(nBlockAlign);                  /* nBlockSize */
+	     OUTSHRT(1);                            /* nFramesPerBlock */
+	     OUTSHRT(0);                            /* nCodecDelay */
 	 
-	 OUTSHRT(AVI->track[j].a_bits);          /* BitsPerSample */
-	 
+	 } else {
+
+	     OUTLONG(16);                   /* # of bytes to follow */
+	     OUTSHRT(AVI->track[j].a_fmt);           /* Format */
+	     OUTSHRT(AVI->track[j].a_chans);         /* Number of channels */
+	     OUTLONG(AVI->track[j].a_rate);          /* SamplesPerSec */
+	     //ThOe/tibit
+	     OUTLONG(1000*AVI->track[j].mp3rate/8);
+	     OUTSHRT(sampsize/4);                    /* BlockAlign */
+	     OUTSHRT(AVI->track[j].a_bits);          /* BitsPerSample */
+
+	 }
 	 /* Finish stream list, i.e. put number of bytes in the list to proper pos */
        }
        long2str(AVI_header+strl_start-4,nhb-strl_start);
@@ -776,26 +983,34 @@ static int avi_close_output_file(avi_t *AVI)
 #ifdef INFO_LIST
    OUT4CC ("LIST");
    
-   //FIXME
+   info_start_pos = nhb;
    info_len = MAX_INFO_STRLEN + 12;
-   OUTLONG(info_len);
+   OUTLONG(info_len); // rewritten later
    OUT4CC ("INFO");
 
-//   OUT4CC ("INAM");
-//   OUTLONG(MAX_INFO_STRLEN);
-
-//   sprintf(id_str, "\t");
-//   memset(AVI_header+nhb, 0, MAX_INFO_STRLEN);
-//   memcpy(AVI_header+nhb, id_str, strlen(id_str));
-//   nhb += MAX_INFO_STRLEN;
-
    OUT4CC ("ISFT");
-   OUTLONG(MAX_INFO_STRLEN);
+   //OUTLONG(MAX_INFO_STRLEN);
+   memset(id_str, 0, MAX_INFO_STRLEN);
 
    sprintf(id_str, "%s-%s", PACKAGE, VERSION);
-   memset(AVI_header+nhb, 0, MAX_INFO_STRLEN);
-   memcpy(AVI_header+nhb, id_str, strlen(id_str));
-   nhb += MAX_INFO_STRLEN;
+   real_id_len = id_len = strlen(id_str)+1;
+   if (id_len&1) id_len++;
+
+   OUTLONG(real_id_len);
+
+   memset(AVI_header+nhb, 0, id_len);
+   memcpy(AVI_header+nhb, id_str, id_len);
+   nhb += id_len;
+
+   //if (AVI->comment_fd>0) {
+       info_len = avi_parse_comments (AVI->comment_fd, AVI_header+nhb, HEADERBYTES - nhb - 8 - 12);
+   //}
+   if (info_len <= 0) info_len=0;
+
+   // write correct len
+   long2str(AVI_header+info_start_pos, info_len + id_len + 4+4+4);
+
+   nhb += info_len;
 
 //   OUT4CC ("ICMT");
 //   OUTLONG(MAX_INFO_STRLEN);
@@ -866,13 +1081,19 @@ static int avi_write_data(avi_t *AVI, char *data, unsigned long length, int audi
    int n;
 
    unsigned char astr[5];
-
+   
+   // transcode core itself checks for the size -- unneeded and 
+   // does harm to xvid 2pass encodes where the first pass can get 
+   // _very_ large -- tibit.
+   
+#if 0 
    /* Check for maximum file length */
    
    if ( (AVI->pos + 8 + length + 8 + (AVI->n_idx+1)*16) > AVI_MAX_LEN ) {
      AVI_errno = AVI_ERR_SIZELIM;
      return -1;
    }
+#endif
    
    /* Add index entry */
 
@@ -880,7 +1101,7 @@ static int avi_write_data(avi_t *AVI, char *data, unsigned long length, int audi
    sprintf(astr, "0%1dwb", AVI->aptr+1);
 
    if(audio)
-     n = avi_add_index_entry(AVI,astr,0x00,AVI->pos,length);
+     n = avi_add_index_entry(AVI,astr,0x10,AVI->pos,length);
    else
      n = avi_add_index_entry(AVI,"00db",((keyframe)?0x10:0x0),AVI->pos,length);
    
@@ -900,7 +1121,7 @@ static int avi_write_data(avi_t *AVI, char *data, unsigned long length, int audi
 
 int AVI_write_frame(avi_t *AVI, char *data, long bytes, int keyframe)
 {
-  unsigned long pos;
+  off_t pos;
   
   if(AVI->mode==AVI_MODE_READ) { AVI_errno = AVI_ERR_NOT_PERM; return -1; }
   
@@ -931,6 +1152,7 @@ int AVI_write_audio(avi_t *AVI, char *data, long bytes)
 
    if( avi_write_data(AVI,data,bytes,1,0) ) return -1;
    AVI->track[AVI->aptr].audio_bytes += bytes;
+   AVI->track[AVI->aptr].audio_chunks++;
    return 0;
 }
 
@@ -938,6 +1160,7 @@ int AVI_write_audio(avi_t *AVI, char *data, long bytes)
 int AVI_append_audio(avi_t *AVI, char *data, long bytes)
 {
 
+    // won't work for >2gb
   long i, length, pos;
   unsigned char c[4];
 
@@ -1002,6 +1225,25 @@ int AVI_get_audio_track(avi_t *AVI)
     return(AVI->aptr);
 }
 
+void AVI_set_audio_vbr(avi_t *AVI, long is_vbr)
+{
+    AVI->track[AVI->aptr].a_vbr = is_vbr;
+}
+
+long AVI_get_audio_vbr(avi_t *AVI)
+{
+    return(AVI->track[AVI->aptr].a_vbr);
+}
+
+void AVI_set_comment_fd(avi_t *AVI, int fd)
+{
+    AVI->comment_fd = fd;
+}
+int  AVI_get_comment_fd(avi_t *AVI)
+{
+    return AVI->comment_fd;
+}
+
 
 /*******************************************************************
  *                                                                 *
@@ -1023,6 +1265,9 @@ int AVI_close(avi_t *AVI)
 
    /* Even if there happened an error, we first clean up */
 
+   if (AVI->comment_fd>0)
+       close(AVI->comment_fd);
+   AVI->comment_fd = -1;
    close(AVI->fdes);
    if(AVI->idx) free(AVI->idx);
    if(AVI->video_index) free(AVI->video_index);
@@ -1042,6 +1287,80 @@ int AVI_close(avi_t *AVI)
    return 0; \
 }
 
+avi_t *AVI_open_input_indexfile(char *filename, int getIndex, char *indexfile)
+{
+  avi_t *AVI=NULL;
+  
+  /* Create avi_t structure */
+  
+  AVI = (avi_t *) malloc(sizeof(avi_t));
+  if(AVI==NULL)
+    {
+      AVI_errno = AVI_ERR_NO_MEM;
+      return 0;
+    }
+  memset((void *)AVI,0,sizeof(avi_t));
+  
+  AVI->mode = AVI_MODE_READ; /* open for reading */
+  
+  /* Open the file */
+  
+  AVI->fdes = open(filename,O_RDONLY);
+  if(AVI->fdes < 0)
+    {
+      AVI_errno = AVI_ERR_OPEN;
+      free(AVI);
+      return 0;
+    }
+  
+  AVI->index_file=strdup(indexfile);
+  AVI_errno = 0;
+  avi_parse_input_file(AVI, getIndex);
+
+  if (AVI != NULL && !AVI_errno) {
+      AVI->aptr=0; //reset  
+  }
+
+  if (AVI_errno)
+      return AVI=NULL;
+  else
+      return AVI;
+}
+
+avi_t *AVI_open_indexfd(int fd, int getIndex, char *indexfile)
+{
+  avi_t *AVI=NULL;
+  
+  /* Create avi_t structure */
+  
+  AVI = (avi_t *) malloc(sizeof(avi_t));
+  if(AVI==NULL)
+    {
+      AVI_errno = AVI_ERR_NO_MEM;
+      return 0;
+    }
+  memset((void *)AVI,0,sizeof(avi_t));
+  
+  AVI->mode = AVI_MODE_READ; /* open for reading */
+  
+  // file alread open
+  AVI->fdes = fd;
+  
+  AVI->index_file=strdup(indexfile);
+  AVI_errno = 0;
+  avi_parse_input_file(AVI, getIndex);
+
+  if (AVI != NULL && !AVI_errno) {
+      AVI->aptr=0; //reset
+  }
+  
+  if (AVI_errno)
+      return AVI=NULL;
+  else
+      return AVI;
+}
+
+
 avi_t *AVI_open_input_file(char *filename, int getIndex)
 {
   avi_t *AVI=NULL;
@@ -1113,9 +1432,120 @@ avi_t *AVI_open_fd(int fd, int getIndex)
       return AVI;
 }
 
+// transcode-0.6.8
+// reads a file generated by aviindex and builds the index out of it.
+
+int avi_parse_index_from_file(avi_t *AVI, char *filename)
+{
+    char data[100]; // line buffer
+    FILE *fd = NULL; // read from
+    off_t pos, len, f_pos, tot_chunks[AVI_MAX_TRACKS];
+    int key=0, type;
+    int vid_chunks=0, aud_chunks[AVI_MAX_TRACKS];
+    long line_count=0;
+    char *c, d;
+    int i,j;
+
+    for (i=0; i<AVI_MAX_TRACKS; i++) aud_chunks[i] = 0;
+    i=0;
+
+    if (AVI->video_index)  // already have an index
+	return 0;
+
+    if (!(fd = fopen(filename, "r"))) { perror ("avi_parse_index_from_file: fopen"); return -1; }
+
+    // read header
+    fgets(data, 100, fd);
+
+    if ( strncasecmp(data, "AVIIDX1", 7) != 0) {
+	fprintf(stderr, "%s: Not an AVI index file\n", filename);
+	return -1;
+    }
+
+    // read comment
+    fgets(data, 100, fd);
+    f_pos = ftell(fd);
+    while (fgets(data, 100, fd)) {
+	d = data[1];
+	if        (d == '0') {
+	    vid_chunks++;
+	} else if (d == '1' || d == '2' || d == '3' || d == '4' ||
+		   d == '5' || d == '6' || d == '7' || d == '8'   ) {
+	    aud_chunks[d-'1']++;
+	} else 
+	    continue;
+
+	line_count++;
+    }
+
+    AVI->video_frames = vid_chunks;
+    for(j=0; j<AVI->anum; ++j) AVI->track[j].audio_chunks = aud_chunks[j];
+
+    if(AVI->video_frames==0) ERR_EXIT(AVI_ERR_NO_VIDS);
+    AVI->video_index = (video_index_entry *) malloc(vid_chunks*sizeof(video_index_entry));
+    if(AVI->video_index==0) ERR_EXIT(AVI_ERR_NO_MEM);
+
+    for(j=0; j<AVI->anum; ++j) {
+	if(AVI->track[j].audio_chunks) {
+	    AVI->track[j].audio_index = (audio_index_entry *) malloc(aud_chunks[j]*sizeof(audio_index_entry));
+	    if(AVI->track[j].audio_index==0) ERR_EXIT(AVI_ERR_NO_MEM);
+	}
+    }   
+
+    // reset after header
+    fseek(fd, f_pos, SEEK_SET);
+
+    vid_chunks = 0;
+    for(j=0; j<AVI->anum; ++j) aud_chunks[j] = tot_chunks[j] = 0;
+
+    while (fgets(data, 100, fd)) {
+	// this is very slow
+	// sscanf(data, "%*s %d %*d %*d %lld %lld %d %*f", &type,  &pos, &len, &key);
+	c     = strchr (data, ' ');
+	type  = strtol(c+1, &c, 10);
+	//ch    = strtol(c+1, &c, 10);
+	c = strchr(c+1, ' ');
+	//chtype= strtol(c+1, &c, 10);
+	c = strchr(c+1, ' ');
+	pos   = strtoll(c+1, &c, 10);
+	len   = strtol(c+1, &c, 10);
+	key   = strtol(c+1, &c, 10);
+	//ms    = strtod(c+1, NULL);
+	i = type-1;
+
+	switch (i) {
+	    case 0: // video
+		AVI->video_index[vid_chunks].key = (off_t)key;
+		AVI->video_index[vid_chunks].pos = pos+8;
+		AVI->video_index[vid_chunks].len = len;
+		vid_chunks++;
+		break;
+	    case 1: case 2: case 3: case 4:
+	    case 5: case 6: case 7: case 8:
+		j=i-1;
+		AVI->track[j].audio_index[aud_chunks[j]].pos = pos+8;
+		AVI->track[j].audio_index[aud_chunks[j]].len = len;
+		AVI->track[j].audio_index[aud_chunks[j]].tot = tot_chunks[j];
+		tot_chunks[j] += AVI->track[j].audio_index[aud_chunks[j]].len;
+		aud_chunks[j]++;
+		break;
+	    default:
+		continue;
+	}
+
+    }
+    for(j=0; j<AVI->anum; ++j) AVI->track[j].audio_bytes = tot_chunks[j];
+
+    fclose (fd);
+
+
+    return 0;
+}
+
 int avi_parse_input_file(avi_t *AVI, int getIndex)
 {
-  long i, n, rate, scale, idx_type;
+  long i, rate, scale, idx_type;
+  off_t n;
   unsigned char *hdrl_data;
   long header_offset=0, hdrl_len=0;
   long nvi, nai[AVI_MAX_TRACKS], ioff;
@@ -1128,6 +1558,7 @@ int avi_parse_input_file(avi_t *AVI, int getIndex)
   //  int auds_strf_seen = 0;
   int num_stream = 0;
   char data[256];
+  off_t oldpos=-1, newpos=-1;
   
   /* Read first 12 bytes and check that this is an AVI file */
 
@@ -1142,9 +1573,16 @@ int avi_parse_input_file(avi_t *AVI, int getIndex)
 
    hdrl_data = 0;
 
+
    while(1)
    {
       if( avi_read(AVI->fdes,data,8) != 8 ) break; /* We assume it's EOF */
+      newpos=lseek(AVI->fdes,0,SEEK_CUR);
+      if(oldpos==newpos) {
+	      /* This is a broken AVI stream... */
+	      return -1;
+      }
+      oldpos=newpos;
 
       n = str2ulong(data+4);
       n = PAD_EVEN(n);
@@ -1312,6 +1750,18 @@ int avi_parse_input_file(avi_t *AVI, int getIndex)
 
    /* get index if wanted */
 
+   if(AVI->index_file && !getIndex) {
+       int ret;
+
+       ret = avi_parse_index_from_file(AVI, AVI->index_file);
+
+       /* Reposition the file */
+   
+       lseek(AVI->fdes,AVI->movi_start,SEEK_SET);
+       AVI->video_pos = 0;
+       return (ret);
+
+   }
    if(!getIndex) return(0);
 
    /* if the file has an idx1, check if this is relative
@@ -1321,7 +1771,7 @@ int avi_parse_input_file(avi_t *AVI, int getIndex)
 
    if(AVI->idx)
    {
-      long pos, len;
+      off_t pos, len;
 
       /* Search the first videoframe in the idx1 and look where
          it is in the file */
@@ -1621,6 +2071,27 @@ long AVI_read_frame(avi_t *AVI, char *vidbuf, int *keyframe)
    return n;
 }
 
+long AVI_get_audio_position_index(avi_t *AVI)
+{
+   if(AVI->mode==AVI_MODE_WRITE) { AVI_errno = AVI_ERR_NOT_PERM; return -1; }
+   if(!AVI->track[AVI->aptr].audio_index) { AVI_errno = AVI_ERR_NO_IDX;   return -1; }
+
+   return (AVI->track[AVI->aptr].audio_posc);
+}
+
+int AVI_set_audio_position_index(avi_t *AVI, long indexpos)
+{
+   if(AVI->mode==AVI_MODE_WRITE) { AVI_errno = AVI_ERR_NOT_PERM; return -1; }
+   if(!AVI->track[AVI->aptr].audio_index)         { AVI_errno = AVI_ERR_NO_IDX;   return -1; }
+   if(indexpos > AVI->track[AVI->aptr].audio_chunks)     { AVI_errno = AVI_ERR_NO_IDX;   return -1; }
+
+   AVI->track[AVI->aptr].audio_posc = indexpos;
+   AVI->track[AVI->aptr].audio_posb = 0;
+
+   return 0;
+}
+
+
 int AVI_set_audio_position(avi_t *AVI, long byte)
 {
    long n0, n1, n;
@@ -1652,7 +2123,8 @@ int AVI_set_audio_position(avi_t *AVI, long byte)
 
 long AVI_read_audio(avi_t *AVI, char *audbuf, long bytes)
 {
-   long nr, pos, left, todo;
+   long nr, left, todo;
+   off_t pos;
 
    if(AVI->mode==AVI_MODE_WRITE) { AVI_errno = AVI_ERR_NOT_PERM; return -1; }
    if(!AVI->track[AVI->aptr].audio_index)         { AVI_errno = AVI_ERR_NO_IDX;   return -1; }
@@ -1688,6 +2160,39 @@ long AVI_read_audio(avi_t *AVI, char *audbuf, long bytes)
    return nr;
 }
 
+long AVI_read_audio_chunk(avi_t *AVI, char *audbuf)
+{
+   long left;
+   off_t pos;
+
+   if(AVI->mode==AVI_MODE_WRITE) { AVI_errno = AVI_ERR_NOT_PERM; return -1; }
+   if(!AVI->track[AVI->aptr].audio_index)         { AVI_errno = AVI_ERR_NO_IDX;   return -1; }
+
+   if (AVI->track[AVI->aptr].audio_posc+1>AVI->track[AVI->aptr].audio_chunks) return -1;
+
+   left = AVI->track[AVI->aptr].audio_index[AVI->track[AVI->aptr].audio_posc].len - AVI->track[AVI->aptr].audio_posb;
+   
+   if (audbuf == NULL) return left;
+   
+   if(left==0) {
+       AVI->track[AVI->aptr].audio_posc++;
+       AVI->track[AVI->aptr].audio_posb = 0;
+       return 0;
+   }
+
+   pos = AVI->track[AVI->aptr].audio_index[AVI->track[AVI->aptr].audio_posc].pos + AVI->track[AVI->aptr].audio_posb;
+   lseek(AVI->fdes, pos, SEEK_SET);
+   if (avi_read(AVI->fdes,audbuf,left) != left)
+   {
+      AVI_errno = AVI_ERR_READ;
+      return -1;
+   }
+   AVI->track[AVI->aptr].audio_posc++;
+   AVI->track[AVI->aptr].audio_posb = 0;
+
+   return left;
+}
+
 /* AVI_read_data: Special routine for reading the next audio or video chunk
                   without having an index of the file. */
 
@@ -1706,7 +2211,7 @@ int AVI_read_data(avi_t *AVI, char *vidbuf, long max_vidbuf,
  *   -2 = audio buffer too small
  */
 
-   int n;
+   off_t n;
    char data[8];
  
    if(AVI->mode==AVI_MODE_WRITE) return 0;
