@@ -60,7 +60,7 @@
 #include "vbr.h"
 
 #define MOD_NAME    "export_divx4.so"
-#define MOD_VERSION "v0.3.8 (2003-04-08)"
+#define MOD_VERSION "v0.3.9 (2003-06-09)"
 #define MOD_CODEC   "(video) DivX 4.x/5.x | (audio) MPEG/AC3/PCM"
 
 #define MOD_PRE divx4
@@ -442,6 +442,9 @@ MOD_open
     AVI_set_video(vob->avifile_out, vob->ex_v_width, vob->ex_v_height, 
 		  vob->fps, "DIVX");
 
+    if (vob->avi_comment_fd>0)
+	AVI_set_comment_fd(vob->avifile_out, vob->avi_comment_fd);
+
     //do not force key frame at the very beginning of encoding, since
     //first frame will be a key fame anayway. Therefore key.quantizer
     //is well defined for any frame to follow
