@@ -48,7 +48,7 @@
 # endif
 #endif
 
-#ifndef __FreeBSD__ /* We have malloc() in stdlib.h */
+#if !defined(__FreeBSD__) && !defined(__APPLE__)
 #include <malloc.h>
 #endif
 
@@ -72,12 +72,15 @@
  ****************************************************************************/
 
 #define MOD_NAME    "export_xvidcvs.so"
-#define MOD_VERSION "v0.3.9 (2003-04-08)"
+#define MOD_VERSION "v0.3.10 (2003-06-09)"
 #define MOD_CODEC  \
 "(video) XviD (CVS 2003-03-16)  | (audio) MPEG/AC3/PCM"
 #define MOD_PRE xvidcvs_ 
 #include "export_def.h"
 
+extern pthread_mutex_t delay_video_frames_lock;
+extern int video_frames_delay;
+
 /*****************************************************************************
  * Local data
  ****************************************************************************/
@@ -176,6 +179,7 @@ MOD_init
 		}
 
 		/* Load the codec */
+		//if(xvid2_init("/data/scr/comp/video/xvid/xvid_20030610/xvidcore/build/generic")<0) {
 		if(xvid2_init(vob->mod_path)<0) {
 			fprintf(stderr, "Failed to init XviD codec");
 			return(TC_EXPORT_ERROR); 
@@ -372,6 +376,9 @@ MOD_open
 		AVI_set_video(vob->avifile_out, vob->ex_v_width,
 			      vob->ex_v_height, vob->fps, "XVID");
 
+		if (vob->avi_comment_fd>0)
+		    AVI_set_comment_fd(vob->avifile_out, vob->avi_comment_fd);
+
 		return(0);
 
 	}
@@ -388,6 +395,7 @@ MOD_open
 MOD_encode
 {
 	int xerr;
+	static int fr=0;
 
 	XVID_ENC_FRAME xframe;
 	XVID_ENC_STATS xstats;
@@ -454,10 +462,29 @@ MOD_encode
 	 */
 	if(xframe.intra == 1) tc_outstream_rotate();
 
-	/* Write bitstream */
-	if(AVI_write_frame(avifile, buffer, xframe.length, xframe.intra == 1) < 0) {
+	if (verbose & TC_DEBUG) { // tibit
+	    fprintf(stderr, "[%s] Got frame #%04d type ", MOD_NAME, fr++);
+	    switch (xframe.intra) {
+		case 0: fprintf(stderr, "|%c|-frame", 'P'); break;
+		case 1: fprintf(stderr, "|%c|-frame", 'I'); break;
+		case 2: fprintf(stderr, "|%c|-frame", 'B'); break;
+		case 3: fprintf(stderr, "|%c|-frame", 'S'); break;
+		case 4: fprintf(stderr, "|%c|-frame", 'p'); break;
+		case 5: fprintf(stderr, "|%c|-frame", 's'); break;
+	    }
+	    fprintf(stderr, " len (%07d)\n", xframe.length);
+	}
+
+	if(xframe.intra == 5) { // skipped frame
+	    pthread_mutex_lock(&delay_video_frames_lock);
+	    video_frames_delay++;
+	    pthread_mutex_unlock(&delay_video_frames_lock);
+	} else {
+	    /* Write bitstream */
+	    if(AVI_write_frame(avifile, buffer, xframe.length, xframe.intra == 1) < 0) {
 		fprintf(stderr, "avi video write error");
 		return(TC_EXPORT_ERROR); 
+	    }
 	}
     
 	return(0);
