@@ -39,7 +39,7 @@
 #include <sys/shm.h>
 
 #ifdef __FreeBSD__ /* We don't have on_exit() */
-dv_display_t 	*dv_dpy_on_exit_hack = NULL;
+xv_display_t 	*xv_dpy_on_exit_hack = NULL;
 #endif
 
 #define XV_FORMAT_MASK		0x03
@@ -127,7 +127,7 @@ void xv_display_exit(xv_display_t *dv_dpy) {
   free(dv_dpy);
   dv_dpy = NULL;
 #ifdef __FreeBSD__
-  dv_dpy_on_exit_hack = NULL;
+  xv_dpy_on_exit_hack = NULL;
 #endif
 } // xv_display_exit
 
@@ -686,7 +686,7 @@ int xv_display_init(xv_display_t *dv_dpy, int *argc, char ***argv, int width, in
   }
   
 #ifdef __FreeBSD__
-  dv_dpy_on_exit_hack = dv_dpy;
+  xv_dpy_on_exit_hack = dv_dpy;
   atexit(xv_display_on_exit_hack_handler);
 #else
   on_exit(xv_display_exit_handler, dv_dpy);
