@@ -51,7 +51,7 @@ static int sfd=0;
 
 static double fps;
 
-static pthread_t thread;
+static pthread_t thread=(pthread_t)0;
 
 static int clone_read_thread_flag=0;
 
@@ -75,7 +75,7 @@ int clone_init(FILE *fd)
     return(-1);
   }
   
-  if(verbose & TC_DEBUG) fprintf(stderr, "(%s) reading video frame sync data from %s\n", __FILE__, logfile);
+  if(verbose & TC_DEBUG) fprintf(stderr, "\n(%s) reading video frame sync data from %s\n", __FILE__, logfile);
   
   // allocate space, assume max buffer size
   
@@ -273,8 +273,11 @@ void clone_close()
     void *status;
 
     // cancel the thread
-    pthread_cancel(thread);
-    pthread_join(thread, &status);
+    if (thread) {
+      pthread_cancel(thread);
+      pthread_join(thread, &status);
+      thread = (pthread_t)0;
+    }
 
     //reentrance safe
   
@@ -292,6 +295,9 @@ void clone_close()
 	free(logfile);
 	sfd=0;
     }
+
+    if (pfd) pclose(pfd);
+    pfd = NULL;
 }
 
 char *clone_fifo() 
@@ -316,7 +322,7 @@ char *clone_fifo()
   } 
 #endif
 
-  return(name);
+  return(logfile);
 }
 
 void clone_read_thread()
