@@ -59,6 +59,7 @@ static void *XviD_decore_handle=NULL;
 static void *handle=NULL;
 
 static int global_colorspace;
+static int done_seek=0;
 
 static int x_dim, y_dim;
 
@@ -156,11 +157,26 @@ MOD_open
 
   if(param->flag == TC_VIDEO) {
     
-    if(avifile==NULL) 
-      if(NULL == (avifile = AVI_open_input_file(vob->video_in_file,1))){
-	AVI_print_error("avi open error");
-	return(TC_IMPORT_ERROR); 
+    if(avifile==NULL)  {
+      if(vob->nav_seek_file) {
+	if(NULL == (avifile = AVI_open_input_indexfile(vob->video_in_file,0,vob->nav_seek_file))){
+	  AVI_print_error("avi open error");
+	  return(TC_IMPORT_ERROR); 
+	} 
+      } else {
+	if(NULL == (avifile = AVI_open_input_file(vob->video_in_file,1))){
+	  AVI_print_error("avi open error");
+	  return(TC_IMPORT_ERROR); 
+	} 
       }
+    }
+   
+    // vob->offset contains the last keyframe
+    if (!done_seek && vob->vob_offset>0) {
+	AVI_set_video_position(avifile, vob->vob_offset);
+	done_seek=1;
+    }
+
     
     codec_str = AVI_video_compressor(avifile);
     if(strlen(codec_str)==0) {
@@ -177,6 +193,7 @@ MOD_open
     }
 
     //load the codec
+    //if(xvid2_init("/data/scr/comp/video/xvid/xvid_20030610/xvidcore/build/generic")<0) {
     if(xvid2_init(vob->mod_path)<0) {
       printf("failed to init Xvid codec");
       return(TC_IMPORT_ERROR); 
@@ -194,7 +211,7 @@ MOD_open
     xerr = XviD_decore(NULL, XVID_DEC_CREATE, &xparam, NULL);
 
     if(xerr == XVID_ERR_FAIL) {
-      printf("codec open error");
+      printf("codec open error\n");
       return(TC_EXPORT_ERROR); 
     }
     XviD_decore_handle=xparam.handle;
@@ -270,7 +287,7 @@ MOD_decode {
     xerr = XviD_decore(XviD_decore_handle, XVID_DEC_DECODE, &xframe, NULL);
     if (xerr != XVID_ERR_OK) {
       fprintf(stderr, "[%s] frame decoding failed. Perhaps you're trying to " \
-             "decode MS-MPEG4v3 (aka DivX ;-) aka DivX3)?", MOD_NAME);
+             "decode MS-MPEG4v3 (aka DivX ;-) aka DivX3)?\n", MOD_NAME);
       return(TC_IMPORT_ERROR);
     }
     
@@ -293,10 +310,11 @@ MOD_close
 
     xerr = XviD_decore(XviD_decore_handle, XVID_DEC_DESTROY, NULL, NULL);
     if (xerr == XVID_ERR_FAIL)
-      printf("encoder close error");
+      printf("encoder close error\n");
 
     //remove codec
     dlclose(handle);
+    done_seek=0;
 
     return(0);
   }
