@@ -62,12 +62,13 @@ void usage(int status)
   fprintf(stderr,"\t -i file           input file [stdin]\n");
   fprintf(stderr,"\t -x codec          source codec (required)\n");
   fprintf(stderr,"\t -g wxh            stream frame size [autodetect]\n");
-  fprintf(stderr,"\t -y codec          output raw stream codec [rgb]\n");
+  fprintf(stderr,"\t -y format         output raw stream format [rgb]\n");
   fprintf(stderr,"\t -Q mode           decoding quality (0=fastest-5=best) [%d]\n", VQUALITY);
   fprintf(stderr,"\t -d mode           verbosity mode\n");
   fprintf(stderr,"\t -s c,f,r          audio gain for ac3 downmixing [1,1,1]\n");
   fprintf(stderr,"\t -A n              A52 decoder flag [0]\n");
   fprintf(stderr,"\t -C s,e            decode only from start to end ((V) frames/(A) bytes) [all]\n");
+  fprintf(stderr,"\t -Y                use libdv YUY2 decoder mode\n");
   fprintf(stderr,"\t -v                print version\n");
 
   exit(status);
@@ -103,7 +104,7 @@ int main(int argc, char *argv[])
     ipipe.frame_limit[0]=0; 
     ipipe.frame_limit[1]=LONG_MAX; 
 
-    while ((ch = getopt(argc, argv, "Q:d:x:i:a:g:vy:s:C:A:?h")) != -1) {
+    while ((ch = getopt(argc, argv, "Q:d:x:i:a:g:vy:s:YC:A:?h")) != -1) {
 	
 	switch (ch) {
 	    
@@ -155,6 +156,10 @@ int main(int argc, char *argv[])
 	  exit(0);
 	  break;
 
+	case 'Y': 
+	  ipipe.dv_yuy2_mode=1;
+	  break;
+
 	case 's': 
 	  
 	  if(optarg[0]=='-') usage(EXIT_FAILURE);
@@ -239,10 +244,14 @@ int main(int argc, char *argv[])
 
     if(strcmp(format,"yv12")==0) ipipe.format = TC_CODEC_YV12;
 
+    if(strcmp(format,"yuv2")==0) ipipe.format = TC_CODEC_YUV2;
+
     if(strcmp(format,"yuy2")==0) ipipe.format = TC_CODEC_YUY2;
     
     if(strcmp(format,"pcm")==0)  ipipe.format = TC_CODEC_PCM;
 
+    if(strcmp(format,"raw")==0)  ipipe.format = TC_CODEC_RAW;
+
     /* ------------------------------------------------------------ 
      *
      * codec specific section
@@ -299,6 +308,15 @@ int main(int argc, char *argv[])
 	done = 1;
     }
 
+    // MP2
+    if(strcmp(codec,"mp2")==0) {
+	
+	ipipe.codec = TC_CODEC_MP3;
+
+	decode_mp2(&ipipe);
+	done = 1;
+    }
+
 
     // DV
     if(strcmp(codec,"dv")==0) {
@@ -355,6 +373,13 @@ int main(int argc, char *argv[])
       done = 1;
     }
     
+    // MOV 
+    if(strcmp(codec,"mov")==0) {
+      
+      decode_mov(&ipipe);
+      done = 1;
+    }
+    
     if(!done) {
 	fprintf(stderr, "[%s] (pid=%d) unable to handle codec %s\n", EXE, getpid(), codec);
 	exit(1);
