@@ -46,6 +46,9 @@ static pthread_mutex_t export_lock=PTHREAD_MUTEX_INITIALIZER;
 static int force_exit=0;
 static pthread_mutex_t force_exit_lock=PTHREAD_MUTEX_INITIALIZER;
 
+pthread_mutex_t delay_video_frames_lock=PTHREAD_MUTEX_INITIALIZER;
+int video_frames_delay=0;
+
 static int counter_encoding=0;
 static int counter_skipping=0;
 
@@ -154,12 +157,12 @@ int export_init(vob_t *vob, char *a_mod, char *v_mod)
 
    // load export modules
    if((export_ahandle = load_module(((a_mod==NULL)? TC_DEFAULT_EXPORT_AUDIO: a_mod), TC_EXPORT+TC_AUDIO))==NULL) {
-     fprintf(stderr,"(%s) loading audio export module failed\n", __FILE__);
+     tc_warn("(%s) loading audio export module failed", __FILE__);
      return(-1);
    }
 
    if((export_vhandle = load_module(((v_mod==NULL)? TC_DEFAULT_EXPORT_VIDEO: v_mod), TC_EXPORT+TC_VIDEO))==NULL) {
-     fprintf(stderr,"(%s) loading video export module failed\n", __FILE__);
+     tc_warn("(%s) loading video export module failed", __FILE__);
      return(-1);
    }
 
@@ -190,14 +193,14 @@ int export_init(vob_t *vob, char *a_mod, char *v_mod)
     }
     
     if(!cc) {
-      fprintf(stderr, "(%s) audio codec not supported by export module\n", __FILE__); 
+      tc_warn("(%s) audio codec not supported by export module\n", __FILE__);
       return(-1);
     }
 
   } else { 
    
     if(vob->im_a_codec != CODEC_PCM) {
-      fprintf(stderr, "(%s) audio codec not supported by export module\n", __FILE__); 
+      tc_warn("(%s) audio codec not supported by export module\n", __FILE__); 
       return(-1);
     }
   }
@@ -230,14 +233,14 @@ int export_init(vob_t *vob, char *a_mod, char *v_mod)
     }
     
     if(!cc) {
-      fprintf(stderr, "(%s) video codec not supported by export module\n", __FILE__); 
+      tc_warn("(%s) video codec not supported by export module\n", __FILE__); 
       return(-1);
     }
 
   } else {
     
-    if(vob->im_a_codec != CODEC_RGB) {
-      fprintf(stderr, "(%s) video codec not supported by export module\n", __FILE__); 
+    if(vob->im_v_codec != CODEC_RGB) {
+      tc_warn("(%s) video codec not supported by export module\n", __FILE__); 
       return(-1);
     }
   }
@@ -281,13 +284,13 @@ int encoder_init(transfer_t *export_para, vob_t *vob)
   
   export_para->flag = TC_VIDEO;
   if((ret=tcv_export(TC_EXPORT_INIT, export_para, vob))==TC_EXPORT_ERROR) {
-    fprintf(stderr, "(%s) video export module error: init failed\n", __FILE__);
+    tc_warn("(%s) video export module error: init failed", __FILE__);
     return(-1);
   }
   
   export_para->flag = TC_AUDIO;
   if((ret=tca_export(TC_EXPORT_INIT, export_para, vob))==TC_EXPORT_ERROR) {
-    fprintf(stderr, "(%s) audio export module error: init failed\n", __FILE__);
+    tc_warn("(%s) audio export module error: init failed", __FILE__);
     return(-1);
   }
   
@@ -308,13 +311,13 @@ int encoder_open(transfer_t *export_para, vob_t *vob)
   
   export_para->flag = TC_VIDEO;	
   if((ret=tcv_export(TC_EXPORT_OPEN, export_para, vob))==TC_EXPORT_ERROR) {
-    fprintf(stderr, "(%s) video export module error: open failed\n", __FILE__);
+    tc_warn("(%s) video export module error: open failed", __FILE__);
     return(-1);
   }
   
   export_para->flag = TC_AUDIO;
   if((ret=tca_export(TC_EXPORT_OPEN, export_para, vob))==TC_EXPORT_ERROR) {
-    fprintf(stderr, "(%s) audio export module error: open failed\n", __FILE__);
+    tc_warn("(%s) audio export module error: open failed", __FILE__);
     return(-1);
   }
   
@@ -363,13 +366,13 @@ int encoder_stop(transfer_t *export_para)
 
   export_para->flag = TC_VIDEO;
   if((ret=tcv_export(TC_EXPORT_STOP, export_para, NULL))==TC_EXPORT_ERROR) {
-    fprintf(stderr, "(%s) video export module error: stop failed\n", __FILE__);
+    tc_warn("(%s) video export module error: stop failed", __FILE__);
     return(-1);
   }
   
   export_para->flag = TC_AUDIO;
   if((ret=tca_export(TC_EXPORT_STOP, export_para, NULL))==TC_EXPORT_ERROR) {
-    fprintf(stderr, "(%s) audio export module error: stop failed\n", __FILE__);
+    tc_warn("(%s) audio export module error: stop failed", __FILE__);
     return(-1);
   }
   
@@ -397,6 +400,14 @@ void encoder(vob_t *vob, int frame_a, int frame_b)
     int exit_on_encoder_error=0;
     int fill_flag=0;
 
+    static int this_frame_b=0;
+    static int last_frame_b=0;
+
+    if (this_frame_b != frame_b){
+      last_frame_b = this_frame_b;
+      this_frame_b = frame_b;
+    }
+
     counter_encoding=0;
     counter_skipping=0;
 
@@ -543,12 +554,12 @@ void encoder(vob_t *vob, int frame_a, int frame_b)
 	export_para.size   = vptr->video_size;
 	export_para.attributes = vptr->attributes;
 	
-	if(aptr->attributes & TC_FRAME_IS_KEYFRAME) export_para.attributes |= TC_FRAME_IS_KEYFRAME;
+	if(vptr->attributes & TC_FRAME_IS_KEYFRAME) export_para.attributes |= TC_FRAME_IS_KEYFRAME;
 	
 	export_para.flag   = TC_VIDEO;
 
-	if(tcv_export(TC_EXPORT_ENCODE, &export_para, vob)<0) {
-	  fprintf(stderr, "\nerror encoding video frame\n");
+	if( !(vptr->attributes & TC_FRAME_IS_SKIPPED) && tcv_export(TC_EXPORT_ENCODE, &export_para, vob)<0) {
+	  tc_warn("error encoding video frame");
 	  exit_on_encoder_error=1;
 	}
 
@@ -602,14 +613,23 @@ void encoder(vob_t *vob, int frame_a, int frame_b)
 	export_para.attributes = aptr->attributes;
 	export_para.flag   = TC_AUDIO;
 	
-	if(tca_export(TC_EXPORT_ENCODE, &export_para, vob)<0) {
-	  fprintf(stderr, "\nerror encoding audio frame\n");
-	  exit_on_encoder_error=1;
+	if (video_frames_delay>0) {
+	    pthread_mutex_lock(&delay_video_frames_lock);
+	    --video_frames_delay;
+	    pthread_mutex_unlock(&delay_video_frames_lock);
+	    aptr->attributes |= TC_FRAME_IS_CLONED; 
+	    fprintf(stderr, "[%s] Delaying audio (%d)\n", __FILE__, vob->video_frames_delay);
+	} else {
+	    if( !(aptr->attributes & TC_FRAME_IS_SKIPPED) && tca_export(TC_EXPORT_ENCODE, &export_para, vob)<0) {
+		tc_warn("error encoding audio frame\n");
+		exit_on_encoder_error=1;
+	    }
+	
+	    // maybe clone?
+	    aptr->attributes = export_para.attributes;
+	
 	}
 
-	// maybe clone?
-	aptr->attributes = export_para.attributes;
-	
 	pthread_mutex_lock(&abuffer_ex_fill_lock);
 	--abuffer_ex_fill_ctr;
 	pthread_mutex_unlock(&abuffer_ex_fill_lock);
@@ -629,7 +649,7 @@ void encoder(vob_t *vob, int frame_a, int frame_b)
 	
 	// finished?
 	if(fid >= frame_b) {
-	  if(verbose & TC_DEBUG) fprintf(stderr, "(%s) encoder last frame finished\n", __FILE__);
+	  if(verbose & TC_DEBUG) fprintf(stderr, "\n(%s) encoder last frame finished\n", __FILE__);
 	  
 	  return;
 	}
@@ -646,13 +666,13 @@ void encoder(vob_t *vob, int frame_a, int frame_b)
 	  
 	} else {	
 	  
-	  pthread_mutex_lock(&vbuffer_im_fill_lock);
+	  pthread_mutex_lock(&vbuffer_ex_fill_lock);
 	  --vbuffer_ex_fill_ctr;
-	  pthread_mutex_unlock(&vbuffer_im_fill_lock);
+	  pthread_mutex_unlock(&vbuffer_ex_fill_lock);
 	  
-	  pthread_mutex_lock(&abuffer_im_fill_lock);
+	  pthread_mutex_lock(&abuffer_ex_fill_lock);
 	  --abuffer_ex_fill_ctr;
-	  pthread_mutex_unlock(&abuffer_im_fill_lock);
+	  pthread_mutex_unlock(&abuffer_ex_fill_lock);
 	}
 	
 	if(!counter_skipping) {
@@ -665,7 +685,7 @@ void encoder(vob_t *vob, int frame_a, int frame_b)
 	  if(!fill_flag) {
 	    fill_flag=1;
 	  }
-	  counter_print(0, fid, "skipping", startsec, startusec, "/dev/null", vptr->thread_id);
+	  counter_print(last_frame_b, fid, "skipping", startsec, startusec, "/dev/null", vptr->thread_id);
 	}
 	
       } // frame processing loop
