@@ -43,7 +43,8 @@ static char *std_module[] = {"null",
 			     "xml",
 			     "lav",
 			     "lzo",
-			     "ts"
+			     "ts",
+			     "mplayer"
 };
 
 enum _std_module {_null_, 
@@ -62,7 +63,8 @@ enum _std_module {_null_,
 		  _xml_,
 		  _lav_,
 		  _lzo_,
-		  _ts_
+		  _ts_,
+		  _theora_
 };
 
 static int title, verb;
@@ -103,12 +105,16 @@ char *get_audio_module(int f, int flag)
   }
 }
 
-int probe_source_core(probe_info_t *pvob, int range, char *file)
+int probe_source_core(probe_info_t *pvob, int range, char *file, char *nav_seek_file)
 {
   
   FILE *fd;
 
-  if((snprintf(probe_cmd_buf, PMAX_BUF, "tcprobe -B -i \"%s\" -T %d -d %d -H %d", file, title, verb, range)<0)) return(-1);
+  if(nav_seek_file) {
+    if((snprintf(probe_cmd_buf, PMAX_BUF, "tcprobe -B -i \"%s\" -T %d -d %d -H %d -f \"%s\"", file, title, verb, range, nav_seek_file)<0)) return(-1);
+  } else {
+    if((snprintf(probe_cmd_buf, PMAX_BUF, "tcprobe -B -i \"%s\" -T %d -d %d -H %d", file, title, verb, range)<0)) return(-1);
+  }
 
   
   // popen
@@ -163,7 +169,7 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
 
   if(vid_file!=NULL) {
 
-    if(probe_source_core(pvob, range, vid_file)<0) {
+    if(probe_source_core(pvob, range, vid_file, vob->nav_seek_file)<0) {
       *flag = TC_PROBE_ERROR;
 
       if(verbose & TC_DEBUG) printf("(%s) failed to probe video source\n", __FILE__);	
@@ -176,7 +182,7 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
 
   if(aud_file != NULL) {
     //probe audio file
-    if(probe_source_core(paob, range, aud_file)<0) {
+    if(probe_source_core(paob, range, aud_file, vob->nav_seek_file)<0) {
       *flag = TC_PROBE_ERROR;
       if(verbose & TC_DEBUG) printf("(%s) failed to probe audio source\n", __FILE__);	
 
@@ -635,8 +641,20 @@ void probe_source(int *flag, vob_t *vob, int range, char *vid_file, char *aud_fi
 
     break;
 
+  case TC_CODEC_THEORA:
+    if (probe_further_for_codec) {
+	vob->vmod_probed=std_module[_theora_];
+	preset |= TC_VIDEO;
+	if(preset & TC_AUDIO) break;
+    }
+    vob->amod_probed=get_audio_module(vob->fixme_a_codec, vob->has_audio);
+    preset |= TC_AUDIO;
+    break;
+    
   case TC_CODEC_DIVX3:
   case TC_CODEC_DIVX4:
+  case TC_CODEC_DIVX5:
+  case TC_CODEC_XVID:
 
     // if input is ogg, do not probe video
     if (probe_further_for_codec) {
@@ -725,6 +743,12 @@ char *codec2str(int f)
     
   case TC_CODEC_DIVX4:
     return("DivX");
+
+  case TC_CODEC_DIVX5:
+    return("DivX5");
+
+  case TC_CODEC_XVID:
+    return("XviD");
     
   case TC_CODEC_MPEG1:
     return("MPEG-1");
